import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { IDisposable, IReference } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { ExtensionIdentifier } from "@codingame/monaco-vscode-api/vscode/vs/platform/extensions/common/extensions";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { MainThreadDebugVisualization, IDebugVisualization, IDebugVisualizationContext, IExpression, IDebugVisualizationTreeItem, IDebugSession } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debug";
import { VisualizedExpression } from "@codingame/monaco-vscode-88aa9a78-75f5-5aaa-89e9-bbdd580b3b5c-common/vscode/vs/workbench/contrib/debug/common/debugModel";
import { IExtensionService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/extensions/common/extensions.service";
import { IDebugVisualizerService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/debug/common/debugVisualizers.service";
export interface VisualizerHandle {
    id: string;
    extensionId: ExtensionIdentifier;
    provideDebugVisualizers(context: IDebugVisualizationContext, token: CancellationToken): Promise<IDebugVisualization[]>;
    resolveDebugVisualizer(viz: IDebugVisualization, token: CancellationToken): Promise<MainThreadDebugVisualization>;
    executeDebugVisualizerCommand(id: number): Promise<void>;
    disposeDebugVisualizers(ids: number[]): void;
}
export interface VisualizerTreeHandle {
    getTreeItem(element: IDebugVisualizationContext): Promise<IDebugVisualizationTreeItem | undefined>;
    getChildren(element: number): Promise<IDebugVisualizationTreeItem[]>;
    disposeItem(element: number): void;
    editItem?(item: number, value: string): Promise<IDebugVisualizationTreeItem | undefined>;
}
export declare class DebugVisualizer {
    private readonly handle;
    private readonly viz;
    get name(): string;
    get iconPath(): {
        light?: import("@codingame/monaco-vscode-api/vscode/vs/base/common/uri").URI;
        dark: import("@codingame/monaco-vscode-api/vscode/vs/base/common/uri").URI;
    } | undefined;
    get iconClass(): string | undefined;
    constructor(handle: VisualizerHandle, viz: IDebugVisualization);
    resolve(token: CancellationToken): Promise<MainThreadDebugVisualization>;
    execute(): Promise<void>;
}
export declare class DebugVisualizerService implements IDebugVisualizerService {
    private readonly contextKeyService;
    private readonly extensionService;
    private readonly logService;
    readonly _serviceBrand: undefined;
    private readonly handles;
    private readonly trees;
    private readonly didActivate;
    private registrations;
    constructor(contextKeyService: IContextKeyService, extensionService: IExtensionService, logService: ILogService);
    getApplicableFor(variable: IExpression, token: CancellationToken): Promise<IReference<DebugVisualizer[]>>;
    register(handle: VisualizerHandle): IDisposable;
    registerTree(treeId: string, handle: VisualizerTreeHandle): IDisposable;
    getVisualizedNodeFor(treeId: string, expr: IExpression): Promise<VisualizedExpression | undefined>;
    getVisualizedChildren(session: IDebugSession | undefined, treeId: string, treeElementId: number): Promise<IExpression[]>;
    editTreeItem(treeId: string, treeItem: IDebugVisualizationTreeItem, newValue: string): Promise<void>;
    private getVariableContext;
    private processExtensionRegistration;
}
